# AUTOGENERATED FILE


from __future__ import annotations

from typing import List, Literal, Optional

from miniappi.core.models.content import BaseContent
from pydantic import ConfigDict, Field, RootModel


class Props(BaseContent):
    model_config = ConfigDict(
        extra='allow',
    )
    id: str = Field(..., description='ID of the component')
    options: List[str]
    placeholder: Optional[str] = None
    submitText: Optional[str] = None
    contentType: Literal['v0/widgets/DropDown.vue'] = 'v0/widgets/DropDown.vue'


class Root(RootModel[Props]):
    root: Props
