# AUTOGENERATED FILE


from __future__ import annotations

from typing import Literal, Optional

from miniappi.core.models.content import BaseContent
from pydantic import ConfigDict, Field, RootModel


class Props(BaseContent):
    model_config = ConfigDict(
        extra='allow',
    )
    defaultValue: Optional[str] = Field(
        None,
        description='The default value for the input when not controlled by `modelValue`.',
    )
    id: str = Field(..., description='ID of the component')
    inputmode: Optional[
        Literal['none', 'text', 'tel', 'url', 'email', 'numeric', 'decimal', 'search']
    ] = Field(
        None,
        description='Hints at the type of data that might be entered by the user while editing the element or its contents',
    )
    placeholder: Optional[str] = None
    submitText: Optional[str] = None
    contentType: Literal['v0/widgets/InputText.vue'] = 'v0/widgets/InputText.vue'


class Root(RootModel[Props]):
    root: Props
