# AUTOGENERATED FILE


from __future__ import annotations

from typing import Literal

from miniappi.core.models.content import BaseContent
from pydantic import ConfigDict, Field, RootModel


class Props(BaseContent):
    """
    Simple large header
    """

    model_config = ConfigDict(
        extra='allow',
    )
    id: str = Field(..., description='ID of the component')
    text: str
    contentType: Literal['v0/Title.vue'] = 'v0/Title.vue'


class Root(RootModel[Props]):
    root: Props
