# AUTOGENERATED FILE


from __future__ import annotations

from typing import Any, Literal, Optional

from miniappi.core.models.content import BaseContent
from pydantic import ConfigDict, Field, RootModel


class Props(BaseContent):
    model_config = ConfigDict(
        extra='allow',
    )
    body: Optional[Any] = None
    footer: Optional[Any] = None
    header: Optional[Any] = None
    id: str = Field(..., description='ID of the component')
    subtitle: Optional[str] = None
    title: Optional[str] = None
    contentType: Literal['v0/cards/Card.vue'] = 'v0/cards/Card.vue'


class Root(RootModel[Props]):
    root: Props
